@@ -1,4 +1,5 @@
 import ssl
+from typing import Optional
 
 import hpack
 import hyperframe.frame
@@ -47,11 +48,11 @@ async def test_proxy_forwarding():
         assert proxy.connections[0].is_available()
         assert not proxy.connections[0].is_closed()
 
-        # A connection on a forwarding proxy can handle HTTP requests to any host.
+        # A connection on a forwarding proxy can only handle HTTP requests to the same origin.
         assert proxy.connections[0].can_handle_request(
             Origin(b"http", b"example.com", 80)
         )
-        assert proxy.connections[0].can_handle_request(
+        assert not proxy.connections[0].can_handle_request(
             Origin(b"http", b"other.com", 80)
         )
         assert not proxy.connections[0].can_handle_request(
@@ -125,8 +126,8 @@ class HTTP1ThenHTTP2Stream(AsyncMockStream):
     async def start_tls(
         self,
         ssl_context: ssl.SSLContext,
-        server_hostname: str = None,
-        timeout: float = None,
+        server_hostname: Optional[str] = None,
+        timeout: Optional[float] = None,
     ) -> AsyncNetworkStream:
         self._http2 = True
         return self
@@ -134,7 +135,11 @@ class HTTP1ThenHTTP2Stream(AsyncMockStream):
 
 class HTTP1ThenHTTP2Backend(AsyncMockBackend):
     async def connect_tcp(
-        self, host: str, port: int, timeout: float = None, local_address: str = None
+        self,
+        host: str,
+        port: int,
+        timeout: Optional[float] = None,
+        local_address: Optional[str] = None,
     ) -> AsyncNetworkStream:
         return HTTP1ThenHTTP2Stream(list(self._buffer))
 
